package de.ozgcloud.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.ArrayDeque;
import java.util.Deque;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

class RoleHierarchyTest {

	private final RoleHierarchy subRole = RoleHierarchy.builder().roleName(UserRole.VERWALTUNG_USER).build();
	@Spy
	private final RoleHierarchy root = RoleHierarchy.builder()
			.roleName(UserRole.VERWALTUNG_LOESCHEN)
			.subRole(subRole)
			.build();

	@Nested
	class TestGetFlattenedHierarchy {

		@Test
		void shouldCollectSubRoles() {
			root.getFlattenedHierarchy(UserRole.VERWALTUNG_LOESCHEN);

			verify(root).collectSubRoles(eq(root), eq(UserRole.VERWALTUNG_LOESCHEN), any(Deque.class));
		}
	}

	@Nested
	class TestCollectSubRoles {

		private static final String UNKNOWN_ROLE = "unknown";

		private final Deque<String> collectedSubRoles = new ArrayDeque<>();

		@Test
		void shouldAddLast() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_LOESCHEN, collectedSubRoles);

			assertThat(collectedSubRoles).containsExactly(UserRole.VERWALTUNG_LOESCHEN);
		}

		@Test
		void shouldReturnTrue() {
			var result = root.collectSubRoles(root, UserRole.VERWALTUNG_LOESCHEN, collectedSubRoles);

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var result = root.collectSubRoles(root, UNKNOWN_ROLE, collectedSubRoles);

			assertThat(result).isFalse();
		}

		@Test
		void shouldCollectSubRoles() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_USER, collectedSubRoles);

			verify(root).collectSubRoles(subRole, UserRole.VERWALTUNG_USER, collectedSubRoles);
		}

		@Test
		void shouldReturnTrueOnExistingRole() {
			var result = root.collectSubRoles(root, UserRole.VERWALTUNG_USER, collectedSubRoles);

			assertThat(result).isTrue();
		}

		@Test
		void shouldAddLastSubRole() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_USER, collectedSubRoles);

			assertThat(collectedSubRoles).containsExactly(UserRole.VERWALTUNG_LOESCHEN, UserRole.VERWALTUNG_USER);
		}

		@Test
		void shouldRemoveLast() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_LOESCHEN, collectedSubRoles);

			assertThat(collectedSubRoles).containsExactly(UserRole.VERWALTUNG_LOESCHEN);
		}

	}

}