package de.ozgcloud.user.common.callcontext;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import io.grpc.Metadata;

class GrpcUtilTest {

	private static final String KEY_STR = "dummy-bin";
	private static final String VALUE_STR = "dummyKeyValueWithöäü#!,.-?";

	@DisplayName("Create key of")
	@Nested
	class TestCreateKeyOf {

		@Test
		void shouldReturnKey() {
			var key = GrpcUtil.createKeyOf(KEY_STR);

			assertThat(key.originalName()).isEqualTo(KEY_STR);
			assertThat(key.name()).isEqualTo(KEY_STR);
		}
	}

	@DisplayName("Get from headers")
	@Nested
	class TestGetFromHeaders {

		@Test
		void shouldReturnValueIfExists() {
			var value = GrpcUtil.getFromHeaders(KEY_STR, createMetadata());

			assertThat(value).isPresent();
			assertThat(value.get()).contains(VALUE_STR);
		}

		@Test
		void shouldReturnEmptyIfNotExists() {
			var value = GrpcUtil.getFromHeaders(KEY_STR, new Metadata());

			assertThat(value).isEmpty();
		}
	}

	@DisplayName("Get collection")
	@Nested
	class TestGetCollection {

		@Test
		void shouldReturnSingleValueAsList() {
			var valueList = GrpcUtil.getCollection(KEY_STR, createMetadata());

			assertThat(valueList).containsExactly(VALUE_STR);
		}

		@Test
		void shouldReturnMultipleValuesAsList() {
			var metadata = createMetadata();
			metadata.put(GrpcUtil.createKeyOf(KEY_STR), "otherValue-bin".getBytes());

			var valueList = GrpcUtil.getCollection(KEY_STR, metadata);

			assertThat(valueList).containsExactly(VALUE_STR, "otherValue-bin");
		}

		@Test
		void shouldReturnEmptyListIfNotExists() {
			var valueList = GrpcUtil.getCollection(KEY_STR, new Metadata());

			assertThat(valueList).isEmpty();
		}
	}

	private Metadata createMetadata() {
		var metadata = new Metadata();
		metadata.put(GrpcUtil.createKeyOf(KEY_STR), VALUE_STR.getBytes());
		return metadata;
	}
}
